package FCC::Actions::Index::Moddtlpod;
$VERSION = 1.00;
use strict;
use warnings;
use utf8;
use base qw(FCC::Actions::Index::_Super);
use File::Which;

sub dispatch {
    my ($self) = @_;

    # パラメーターを取得
    my $in = $self->get_input_data( ['path'] );
    unless ( $in->{path} ) {
        $self->print_text( 'パラメーターが不正です。', 400 );
        return;
    }

    my $path = $in->{path};

    # パラメーター `path` の値をチェック
    if (  !$path
        || $path !~ /\.pm$/
        || $path =~ /\.\./
        || $path =~ /[^a-zA-Z0-9\/\.\_\-]/ )
    {
        $self->print_text( "パラメーター `path` の値が不正です: ${path}", 400 );
        return;
    }

    # パラメーター `path`` の値が @INC のパスの中かどうかをチェック
    my @path_parts = split( /\//, $path );
    pop(@path_parts);
    my $dpath         = join( '/', @path_parts );
    my $libpath_list  = $self->get_libpath_list();
    my $is_valid_path = 0;
    for my $path ( @{$libpath_list} ) {
        if ( $dpath =~ /^\Q${path}\E/ ) {
            $is_valid_path = 1;
            last;
        }
    }
    unless ($is_valid_path) {
        $self->print_text( "パラメーター `path` の値が不正です: ${path}", 400 );
        return;
    }

    # pm ファイルの存在をチェック
    unless ( -e $path ) {
        $self->print_text( "パラメーター `path` に指定したファイルが見つかりませんでした: ${path}", 404 );
        return;
    }

    # POD を取得
    my $pod2html = $self->find_pod2html();
    unless ($pod2html) {
        $self->print_text(
            "コマンド pod2html がサーバにインストールされていないため、ドキュメントを表示することが出来ませんでした。", 500 );
        return;
    }

    my $html = '';
    eval { $html = `${pod2html} ${path}`; };
    if ($@) {
        $self->print_text( "${pod2html} コマンドの実行に失敗しました: $@", 500 );
        return;
    }

    $self->print_html($html);
}

# pod2html コマンドのパスを調べる
sub find_pod2html {
    my ($self)           = @_;
    my $cmd              = 'pod2html';
    my @search_path_list = (
        '/bin',      '/usr/bin', '/usr/local/bin', '/sbin',
        '/usr/sbin', '/usr/lib', '/usr/local/lib'
    );
    my @path_list = File::Which::where $cmd;
    unless (@path_list) {
        for my $dpath (@search_path_list) {
            my $fpath = "${dpath}/${cmd}";
            if ( -e $fpath && -x $fpath ) {
                push( @path_list, $fpath );
            }
        }
    }
    my $path = shift @path_list;
    return $path;
}

# Perl モジュール検索パス
sub get_libpath_list {
    my ($self) = @_;
    my $this_path = $self->{conf}->{CGI_DIR};
    my @list;
    for my $path (@INC) {
        if ( $path eq '.' ) {
            next;
        }
        unless ( $path =~ /^\Q${this_path}\E/ ) {
            push( @list, $path );
        }
    }
    return \@list;
}

1;
